local DEG_ADDON = _G["DEG_CURRENT_ADDON"]

local function d(...)
  _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]:d(...)
end

local LAM2 = LibStub:GetLibrary("LibAddonMenu-2.0")

local Obj = {
  initialized = false,
  Addon = nil,
}

function Obj:initialize()
  if self.initialized then return end
  
  self.Addon = _G[DEG_ADDON.PACKAGE_NAME].plugins[DEG_ADDON.ADDON_NAME_SHORT]
              
  local optionsPanelConfig  = {
    type = "panel",
    name = "Dryzler's Setz (Gear Swap)",
    displayName = "|c3f95ffDryzler's|r |cEFEBBESetz (Gear Swap)|r",
    author = "|cEFEBBEdryzler.elder-geek.net|r",
    website = "https://dryzler.elder-geek.net/",
    version = self.Addon.versionString,
    slashCommand = "/drysetz",
    registerForRefresh = true,
    registerForDefaults = true,
  }

  local optionsPanel = LAM2:RegisterAddonPanel(self.Addon.name, optionsPanelConfig)
  
  local optionsPanelControls = {}
  
  
  table.insert(optionsPanelControls, {
      type = "header",
      name = GetString(SI_DEG_SI_SETTINGS_ACCOUNTWIDE),
  })  
    
--  table.insert(optionsPanelControls, {
--      type = "header",
--      name = GetString(SI_DEG_SETZ_OPTS_LEVEL_HEADER),
--  })    
  table.insert(optionsPanelControls, {
      type = "description",
      text = GetString(SI_DEG_SETZ_OPTS_LEVEL_DESC),
  })  
      
  table.insert(optionsPanelControls, {
      type = "slider",
      name = GetString(SI_DEG_SETZ_OPTS_ACCOUNT_LEVEL),
      --tooltip = GetString(SI_DEG_SETZ_OPTS_ACCOUNT_LEVEL_TT),
      min = 0,
      max = 1000,
      step = 1,
      getFunc = function()
        return self.Addon.savedVariablesAccount.settings.levelDist 
      end,
      setFunc = function(newValue) 
        self.Addon.savedVariablesAccount.settings.levelDist = newValue
        self.Addon:parseSets();
      end,
      width = "full",
      default = function() return 10 end,
      disabled = function() 
        return false
      end,
  })  
    
  table.insert(optionsPanelControls, {
      type = "description",
      text = GetString(SI_DEG_SETZ_OPTS_LEVEL_DESC2),
  })    
    
  table.insert(optionsPanelControls, {
      type = "header",
      name = GetString(SI_DEG_SI_SETTINGS_THIS_CHAR),      
  })    
    
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_CHAR_ACTIVE),
      default = function()
        self.Addon.savedVariablesChar.settings.levelDist = false
        return false
      end,
      getFunc = function()
        return not self.Addon.savedVariablesChar.settings.levelDist == false 
      end,
      setFunc = function(newValue)
        if newValue then
          self.Addon.savedVariablesChar.settings.levelDist = self.Addon.savedVariablesAccount.settings.levelDist
        else
          self.Addon.savedVariablesChar.settings.levelDist = false
        end
      end,
  })
  
  table.insert(optionsPanelControls, {
      type = "slider",
      name = GetString(SI_DEG_SETZ_OPTS_CHAR_LEVEL),
      --tooltip = GetString(SI_DEG_SETZ_OPTS_CHAR_LEVEL_TT),
      min = 0,
      max = 1000,
      step = 1,
      getFunc = function()
        if self.Addon.savedVariablesChar.settings.levelDist == false then
          return self.Addon.savedVariablesAccount.settings.levelDist
        else
          return self.Addon.savedVariablesChar.settings.levelDist          
        end        
      end,
      setFunc = function(newValue) 
        self.Addon.savedVariablesChar.settings.levelDist = newValue
        self.Addon:parseSets();
      end,
      width = "full",
      --default = function() return 10 end,
      disabled = function()
        if self.Addon.savedVariablesChar.settings.levelDist == false then
          return true
        end
        return false 
      end,
  })
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_CHAR_GEARSWAP),
      default = function()
        self.Addon.savedVariablesChar.settings.gearSwap = false
        return false
      end,
      getFunc = function()
        return not self.Addon.savedVariablesChar.settings.gearSwap == false 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.gearSwap = newValue
        self.Addon.savedVariablesChar.gears = {}
        self.Addon:saveGear()
      end,
  })  
  
  
  --1) 1 hand waffe + schild = wield                        
  --2) 2 x 1 handwaffe = swords2
  --3) 2 handwaffe = bigsword
  --4) zerstörungsstäbe = damagestaff
  --5) heilungsstäbe = healingstaff
  --6) bögen = bow  
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_WIELD),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.wield 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.wield = newValue
        self.Addon:parseSets();
      end,
  })  
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_SWORDS2),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.swords2 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.swords2 = newValue
        self.Addon:parseSets();
      end,
  })
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_BIGSWORD),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.bigsword 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.bigsword = newValue
        self.Addon:parseSets();
      end,
  })  
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_DAMAGESTAFF),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.damagestaff 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.damagestaff = newValue
        self.Addon:parseSets();
      end,
  })        
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_HEALINGSTAFF),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.healingstaff 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.healingstaff = newValue
        self.Addon:parseSets();
      end,
  })  
  
  table.insert(optionsPanelControls, {
      type = "checkbox",
      name = GetString(SI_DEG_SETZ_OPTS_BOW),
      default = function()
        return false
      end,
      getFunc = function()
        return self.Addon.savedVariablesChar.settings.bow 
      end,
      setFunc = function(newValue)
        self.Addon.savedVariablesChar.settings.bow = newValue
        self.Addon:parseSets();
      end,
  })      
  
  
  LAM2:RegisterOptionControls(self.Addon.name, optionsPanelControls)
  
  self.initialized = true
end

_G[_G["DEG_CURRENT_ADDON"].ADDON_NAME.."Settings"] = Obj