local MAJOR, MINOR = "LibSafe", 1.006
local LibSafe, oldminor = LibStub:NewLibrary(MAJOR, MINOR)
if not LibSafe then return end

LibSafe.addonName = ""
LibSafe.addonTable = nil
LibSafe.savedVariablesName = nil
LibSafe.savedVariables = nil
LibSafe.isDeveloper = false

--static function or instance function
function LibSafe:makeTableSafe(tbl, libSafeObject, isDeveloper)
  if not tbl and not self.addonTable then return end
  if not tbl then
    tbl = self.addonTable
  end
  if not libSafeObject then
    libSafeObject = self
    --if not libSafeObject.addonTable -> completely static call
  end
    
  if not tbl.isLibSafedTable then
    tbl.isLibSafedTable = true
    local writtenFunctions = {}
    for k,v in pairs(tbl) do
      if not writtenFunctions[k] then
        writtenFunctions[k] = true 
        if type(v) == 'function' then
          if libSafeObject.addonTable then
            tbl[k] = function(self2, ...)
              local ret = {libSafeObject:callTable(self2, v, ...)}
              table.remove(ret, 1)
              return unpack(ret)
            end
          else
            tbl[k] = function(self2, ...)
              local isDeveloperBefore = LibSafe.isDeveloper
              LibSafe.isDeveloper = isDeveloper
              local ret = {LibSafe:callTable(self2, v, ...)}
              LibSafe.isDeveloper = isDeveloperBefore
              table.remove(ret, 1)
              return unpack(ret)
            end
          end
        end
      end
    end
  end
end


function LibSafe:call(fn, ...)
  if self.isDeveloper then
    return true, fn(self.addonTable, ...)
  end  
  local args = {...}  
  return xpcall(function()
    return fn(self.addonTable, unpack(args))
  end,
  function(errobj)
    self:errorHandler(errobj)
  end)
end


function LibSafe:callFn(fn, ...)
  if self.isDeveloper then
    return true, fn(...)
  end
  local args = {...}
  return xpcall(function()
    return fn(unpack(args))
  end,   
  function(errobj)
    self:errorHandler(errobj)
  end)
end

function LibSafe:callTable(tbl, fn, ...)
  if self.isDeveloper then
    return true, fn(tbl, ...)
  end
  local args = {...}
  return xpcall(function()
    return fn(tbl, unpack(args))
  end,
  function(errobj)
    self:errorHandler(errobj)
  end)
end

function LibSafe.errorHandler(self, errobj)
  _G.d(self.addonName..">|cFF4500error|r: "..(tostring(errobj):gsub("\n.*", "")))  
  if not self.savedVariables then
    if self.savedVariablesName then
      self.savedVariables = ZO_SavedVars:NewAccountWide(self.savedVariablesName, 1, 'LibSafe', {errors = {}})
    end
  end
  if self.savedVariables and self.savedVariables.errors then
    table.insert(self.savedVariables.errors, tostring(errobj))
    while #self.savedVariables.errors > 100 do table.remove(self.savedVariables.errors, 1) end
  end
  return false
end

LibSafe.__index = LibSafe

LibSafe.new = function(addonTable, addonName, savedVariablesName, isDeveloper)
  local self = setmetatable({}, LibSafe)  
  
  self.addonName = addonName
  self.addonTable = addonTable
  self.savedVariablesName = savedVariablesName  
  self.isDeveloper = isDeveloper
  
  return self
end